package com.sun.media.sound;

import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Control;
import java.util.Vector;
import javax.sound.sampled.Line;
import javax.sound.sampled.Mixer;

abstract class AbstractMixer extends AbstractLine implements Mixer
{
    protected static final int PCM = 0;
    protected static final int ULAW = 1;
    protected static final int ALAW = 2;
    private final Mixer.Info mixerInfo;
    protected Line.Info[] sourceLineInfo;
    protected Line.Info[] targetLineInfo;
    private boolean started;
    private boolean manuallyOpened;
    private final Vector sourceLines;
    private final Vector targetLines;
    
    protected AbstractMixer(Mixer.Info mixerInfo, Control[] array, Line.Info[] sourceLineInfo, Line.Info[] targetLineInfo) {
        super(new Line.Info(Mixer.class), null, array);
        this.started = false;
        this.manuallyOpened = false;
        this.sourceLines = new Vector();
        this.targetLines = new Vector();
        this.mixer = this;
        if (array == null) {
            array = new Control[0];
        }
        this.mixerInfo = mixerInfo;
        this.sourceLineInfo = sourceLineInfo;
        this.targetLineInfo = targetLineInfo;
    }
    
    @Override
    public final Mixer.Info getMixerInfo() {
        return this.mixerInfo;
    }
    
    @Override
    public final Line.Info[] getSourceLineInfo() {
        Line.Info[] array = new Line.Info[this.sourceLineInfo.length];
        System.arraycopy(this.sourceLineInfo, 0, array, 0, this.sourceLineInfo.length);
        return array;
    }
    
    @Override
    public final Line.Info[] getTargetLineInfo() {
        Line.Info[] array = new Line.Info[this.targetLineInfo.length];
        System.arraycopy(this.targetLineInfo, 0, array, 0, this.targetLineInfo.length);
        return array;
    }
    
    @Override
    public final Line.Info[] getSourceLineInfo(Line.Info info) {
        Vector<Object> vector = new Vector();
        for (int i = 0; i < this.sourceLineInfo.length; ++i) {
            if (info.matches(this.sourceLineInfo[i])) {
                vector.addElement(this.sourceLineInfo[i]);
            }
        }
        Line.Info[] array = new Line.Info[vector.size()];
        for (int j = 0; j < array.length; ++j) {
            array[j] = (Line.Info)vector.elementAt(j);
        }
        return array;
    }
    
    @Override
    public final Line.Info[] getTargetLineInfo(Line.Info info) {
        Vector<Object> vector = new Vector();
        for (int i = 0; i < this.targetLineInfo.length; ++i) {
            if (info.matches(this.targetLineInfo[i])) {
                vector.addElement(this.targetLineInfo[i]);
            }
        }
        Line.Info[] array = new Line.Info[vector.size()];
        for (int j = 0; j < array.length; ++j) {
            array[j] = (Line.Info)vector.elementAt(j);
        }
        return array;
    }
    
    @Override
    public final boolean isLineSupported(Line.Info info) {
        int i = 0;
        while (i < this.sourceLineInfo.length) {
            if (info.matches(this.sourceLineInfo[i])) {
                return true;
            }
            ++i;
        }
        int j = 0;
        while (j < this.targetLineInfo.length) {
            if (info.matches(this.targetLineInfo[j])) {
                return true;
            }
            ++j;
        }
        return false;
    }
    
    @Override
    public abstract Line getLine(Line.Info p0) throws LineUnavailableException;
    
    @Override
    public abstract int getMaxLines(Line.Info p0);
    
    protected abstract void implOpen() throws LineUnavailableException;
    
    protected abstract void implStart();
    
    protected abstract void implStop();
    
    protected abstract void implClose();
    
    @Override
    public final Line[] getSourceLines() {
        Line[] array;
        synchronized (this.sourceLines) {
            array = new Line[this.sourceLines.size()];
            for (int i = 0; i < array.length; ++i) {
                array[i] = (Line)this.sourceLines.elementAt(i);
            }
        }
        return array;
    }
    
    @Override
    public final Line[] getTargetLines() {
        Line[] array;
        synchronized (this.targetLines) {
            array = new Line[this.targetLines.size()];
            for (int i = 0; i < array.length; ++i) {
                array[i] = (Line)this.targetLines.elementAt(i);
            }
        }
        return array;
    }
    
    @Override
    public final void synchronize(Line[] array, boolean b) {
        throw new IllegalArgumentException("Synchronization not supported by this mixer.");
    }
    
    @Override
    public final void unsynchronize(Line[] array) {
        throw new IllegalArgumentException("Synchronization not supported by this mixer.");
    }
    
    @Override
    public final boolean isSynchronizationSupported(Line[] array, boolean b) {
        return false;
    }
    
    @Override
    public final synchronized void open() throws LineUnavailableException {
        this.open(true);
    }
    
    final synchronized void open(boolean b) throws LineUnavailableException {
        if (!this.isOpen()) {
            this.implOpen();
            this.setOpen(true);
            if (b) {
                this.manuallyOpened = true;
            }
        }
    }
    
    final synchronized void open(Line obj) throws LineUnavailableException {
        if (this.equals(obj)) {
            return;
        }
        if (this.isSourceLine(obj.getLineInfo())) {
            if (!this.sourceLines.contains(obj)) {
                this.open(false);
                this.sourceLines.addElement(obj);
            }
        }
        else if (this.isTargetLine(obj.getLineInfo()) && !this.targetLines.contains(obj)) {
            this.open(false);
            this.targetLines.addElement(obj);
        }
        return;
    }
    
    final synchronized void close(Line obj) {
        if (this.equals(obj)) {
            return;
        }
        this.sourceLines.removeElement(obj);
        this.targetLines.removeElement(obj);
        if (this.sourceLines.isEmpty() && this.targetLines.isEmpty() && !this.manuallyOpened) {
            this.close();
        }
        return;
    }
    
    @Override
    public final synchronized void close() {
        if (this.isOpen()) {
            Line[] sourceLines = this.getSourceLines();
            for (int i = 0; i < sourceLines.length; ++i) {
                sourceLines[i].close();
            }
            Line[] targetLines = this.getTargetLines();
            for (int j = 0; j < targetLines.length; ++j) {
                targetLines[j].close();
            }
            this.implClose();
            this.setOpen(false);
        }
        this.manuallyOpened = false;
    }
    
    final synchronized void start(Line obj) {
        if (this.equals(obj)) {
            return;
        }
        if (!this.started) {
            this.implStart();
            this.started = true;
        }
        return;
    }
    
    final synchronized void stop(Line obj) {
        if (this.equals(obj)) {
            return;
        }
        Vector<Object> vector = (Vector)this.sourceLines.clone();
        for (int i = 0; i < vector.size(); ++i) {
            if ((vector.elementAt(i) instanceof AbstractDataLine)) {
                AbstractDataLine abstractDataLine = (AbstractDataLine)vector.elementAt(i);
                if (abstractDataLine.isStartedRunning() && !abstractDataLine.equals(obj)) {
                    return;
                }
            }
        }
        Vector<Object> vector2 = (Vector)this.targetLines.clone();
        for (int j = 0; j < vector2.size(); ++j) {
            if ((vector2.elementAt(j) instanceof AbstractDataLine)) {
                AbstractDataLine abstractDataLine2 = (AbstractDataLine)vector2.elementAt(j);
                if (abstractDataLine2.isStartedRunning() && !abstractDataLine2.equals(obj)) {
                    return;
                }
            }
        }
        this.started = false;
        this.implStop();
        return;
    }
    
    final boolean isSourceLine(Line.Info info) {
        int i = 0;
        while (i < this.sourceLineInfo.length) {
            if (info.matches(this.sourceLineInfo[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    final boolean isTargetLine(Line.Info info) {
        int i = 0;
        while (i < this.targetLineInfo.length) {
            if (info.matches(this.targetLineInfo[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    final Line.Info getLineInfo(Line.Info info) {
        if (info == null) {
            return null;
        }
        int i = 0;
        while (i < this.sourceLineInfo.length) {
            if (info.matches(this.sourceLineInfo[i])) {
                return this.sourceLineInfo[i];
            }
            ++i;
        }
        int j = 0;
        while (j < this.targetLineInfo.length) {
            if (info.matches(this.targetLineInfo[j])) {
                return this.targetLineInfo[j];
            }
            ++j;
        }
        return null;
    }
}
